      SUBROUTINE TURNPAR ( OPT, R1, R2, T, 
     .                     ALPHA, TB,  
     .                     TC, OMEGA, ANGACC,
     .                     FOUND, IER ) 

C
C     Determine parameters of a spacecraft turn.
C
C     Turn Model Description
C     ----------------------
C     Turn starts from vector direction R1 and stops
C     at vector direction R2. 
C
C     T = 2*TB + TC, where
C        T  = total turn duration
C        TB = spinup and spindown durations (equal)
C        TC = spin coast duration 
C
C     THETA = 2*THETA_B + THETA_C, where
C        THETA   = total turn angle
C        THETA_B = spinup angle = 0.5 * ALPHA * TB*TB
C        THETA_C = coast angle  = ALPHA * TB * TC 
C        ALPHA   = constant angular acceleration at spinup and
C                  spindown.
C       
C     ALPHA = THETA / (TB*(T-TB)), T >= 2*TB
C
C     TB = (T/2) - SQRT( T^2 - 4*THETA/ALPHA) ), (T^2 - 4*THETA/ALPHA) > 0 
C
C
      IMPLICIT           NONE

      INTEGER            OPT 

      DOUBLE PRECISION   R1      ( 3 ) 
      DOUBLE PRECISION   R2      ( 3 ) 
      DOUBLE PRECISION   T 
      DOUBLE PRECISION   ALPHA 
      DOUBLE PRECISION   TB 
      DOUBLE PRECISION   TC 
      DOUBLE PRECISION   OMEGA   ( 3 ) 
      DOUBLE PRECISION   ANGACC  ( 3 )  

      INTEGER            IER	

      LOGICAL            FOUND

C
C     Inputs 
C     ------
C     OPT = Profile calculation option
C           1 = input R1, R2, DT and ACC to find DTB. 
C           2 = input R1, R2, DT and DTB to find ACC. 
C     R1  = Initial vector, spacecraft frame. 
C     R2  = Final vector of turn, spacecraft frame.
C     T   = Total duration of turn, in seconds. 
C
C     Input or Output
C     ---------------
C     ALPHA = Absolute angular acceleration of spinup, in rad/sec^2. 
C     TB    = Duration of spinup phase, in seconds.
C
C     Outputs
C     -------
C     TC      = Duration of unaccelerated coast, in seconds. 
C     OMEGA   = Overall angular velocity components, in rad/sec. 
C     ANGACC  = Boost angular acceleration components.  Negate
C               these for spindown.
C     FOUND   = Logical flag, .TRUE. if value for TB or ACC is found.
C     IER     = If FOUND = .FALSE., returns an error code.

C
C     Local variables.
C

      DOUBLE PRECISION    A
      DOUBLE PRECISION    ANGVEC   ( 3 ) 
      DOUBLE PRECISION    B
      DOUBLE PRECISION    C 
      DOUBLE PRECISION    DISC 
      DOUBLE PRECISION    ROOT1    ( 2 ) 
      DOUBLE PRECISION    ROOT2    ( 2 )  
      DOUBLE PRECISION    THETA 
      DOUBLE PRECISION    VSEP

      INTEGER             I

      LOGICAL             VZERO            

C
C     Set output flags.
C

      FOUND = .FALSE.
      IER   = 0

C
C     Check for bad inputs.
C

      IF ( VZERO( R1 ) .OR. VZERO( R2 ) .OR. (T .LE. 0.D0 ) ) THEN
         IER = 1
         RETURN 
      END IF

C
C     Compute overall turn angle (.GT. 0 and .LE. 180 deg. ).
C

      THETA = VSEP ( R1, R2 )  

      WRITE(*,*) 'r1 = ', R1
      WRITE(*,*) 'r2 = ', R2
      WRITE(*,*) 'theta = ', THETA

      IF ( THETA .EQ. 0.D0 ) THEN
         IER = 2
         RETURN 
      END IF
  
C*************************************************************
C     Boost duration is provided, find angular acceleration.
C 

      IF ( OPT .EQ. 1 ) THEN

         ALPHA = THETA / ( TB * ( T - TB ) )  

C
C     Check for boost duration within bounds.
C

         IF ( (TB*2.D0 .GT. T) .OR. (TB .LT. 0.D0)  ) THEN
            IER = 3 
            RETURN 
         END IF

C*****************************************************************
C     Angular acceleration is provided, determine boost duration.
C

      ELSE IF ( OPT .EQ. 2 ) THEN

C
C     Setup for quadratic formula. 
C

         A = -ALPHA
         B = ALPHA * T
         C = -THETA 

C
C     Check for sign of discriminant.
C

         DISC = B**2 - 4.D0*A*C

         WRITE(*,*) 'alpha = ', ALPHA 
         WRITE(*,*) 'theta = ', THETA
         WRITE(*,*) 't     = ', T
 
         WRITE(*,*) 'a = ', A
         WRITE(*,*) 'b = ', B
         WRITE(*,*) 'c = ', C

         WRITE(*,*) 'disc = ', DISC
    

C
C     Check for imaginary solutions.
C

         IF ( DISC .LT. 0.D0 ) THEN
            IER = 4 
            RETURN
         END IF
 
         CALL RQUAD ( A, B, C, ROOT1, ROOT2 )  

C
C     Use the second root  
C
 
         WRITE(*,*) 'root1 = ', ROOT1
         WRITE(*,*) 'root2 = ', ROOT2

         TB = ROOT1(1)
         TC = T - TB*2.D0

      ELSE

C
C     Correct option value not found
C
         IER = 5         
         RETURN 

      END IF

C
C     Determine coast duration.
C

      TC = T - TB*2.D0

C
C     Obtain angular velocity unit vector in J2000 frame. 
C

      CALL UCRSS ( R1, R2, ANGVEC )   

C
C     Create overall angular velocity vector.
C

      CALL VSCL ( THETA/T, ANGVEC, OMEGA ) 

C
C     Obtain components of spinup angular acceleration in J2000 frame.
C

      DO I = 1, 3
         ANGACC(I) = ALPHA * ANGVEC(I)
      END DO


      FOUND = .TRUE.

      RETURN
      END

         

